Package: PairViz
Version: 1.3.8
Date: 2026-02-16
Authors@R: c(person(given = c("Catherine", "B."),
                        family = "Hurley",
                        role = "aut"),
                 person(given = "R.W.",
                        family = "Oldford",
                        role = "aut"),
                 person(given = "Catherine",
                        family = "Hurley",
                        role = "cre",
                        email = "catherine.hurley@mu.ie"))
Maintainer: Catherine Hurley <catherine.hurley@mu.ie>
Title: Visualization using Graph Traversal
Description: Improving graphics by ameliorating order effects, using Eulerian tours 
    and Hamiltonian decompositions of  graphs. References for the methods presented 
    here are C.B. Hurley and R.W. Oldford (2010) <doi:10.1198/jcgs.2010.09136> and 
    C.B. Hurley and R.W. Oldford (2011) <doi:10.1007/s00180-011-0229-5>.
Depends: TSP,gtools, graph, methods
Suggests: knitr, rmarkdown, Sleuth3, colorspace,multcomp,igraph,
        scales,RColorBrewer,alr4,scagnostics, gclus, irr, seriation
License: GPL-2
VignetteBuilder: knitr
RoxygenNote: 7.3.3
URL: https://cbhurley.github.io/PairViz/
NeedsCompilation: no
Packaged: 2026-02-18 11:26:51 UTC; catherine
Author: Catherine B. Hurley [aut],
  R.W. Oldford [aut],
  Catherine Hurley [cre]
Repository: CRAN
Date/Publication: 2026-02-18 11:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 08:51:31 UTC; unix
